/* IR Receiver Simple Read Demo
 * http://frank.circleofcurrent.com/musicclock/
 * Copyright (c) 2011 Frank Zhao
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

// please see http://www.nongnu.org/avr-libc/user-manual/modules.html for info on these modules

#include <avr/io.h> // allows access to AVR built-in peripherals and registers
#include <avr/interrupt.h> // utilities to implement interrupt routines
#include <avr/pgmspace.h> // allows access to flash memory
#include <util/delay.h> // provides delays
#include <stdio.h> // allows formatted debug output
#include <avr/power.h> // used for setting clock prescaler
#include "usb_serial/usb_serial.h" // provides USB-to-serial functionality

// pin definitions
#define IRCMD_PORTDIR DDRD
#define IRCMD_OUTPUTPORT PORTD
#define IRCMD_INPUTPORT PIND
#define IRCMD_INPUTPIN 4

// global variables, volatile so they can be safely used inside interrupt routines
static volatile uint16_t last_timestamp;
static volatile char has_overflowed;

// output has changed, thus initiating a capture, causing this interrupt
ISR(TIMER1_CAPT_vect)
{
    // calculate using last timestamp
    uint32_t length = (((uint32_t)(ICR1)) + 0xFFFF - last_timestamp) & 0xFFFF;
    last_timestamp = ICR1;
    has_overflowed = 0; // reset timer overflow flag
    
    // output the pulse width    
    if (bit_is_set(IRCMD_INPUTPORT, IRCMD_INPUTPIN))
    {
        printf("ON: %d \t", (uint16_t)length);
        TCCR1B &= ~_BV(ICES1); // switch triggering edge for next capture
    }
    else
    {
        printf("OFF: %d \r\n", (uint16_t)length);
        TCCR1B |= _BV(ICES1); // switch triggering edge for next capture
    }
    
}

// overflow event, two overflows in a row means the remote control is off
ISR(TIMER1_OVF_vect)
{
    if (has_overflowed == 1)
    {
        printf("OFF: infinity\r\n");
        has_overflowed = 2; // send only once
    }
    else
    {
        if (has_overflowed != 2)
        {
            has_overflowed = 1;
        }
    }
}

void tmrcap_init()
{
    has_overflowed = 0;
    last_timestamp = 0;
    
    IRCMD_OUTPUTPORT &= ~_BV(IRCMD_INPUTPIN); // no pull-up
    IRCMD_PORTDIR &= ~_BV(IRCMD_INPUTPIN); // input capture pin as input
    
    TCCR1B = 0b10000010; // start timer, enable input capture noise canceller, trigger on falling edge
    // prescaler is set to "div by 8"
    
    TIMSK1 = _BV(ICIE1) | _BV(TOIE1); // enable timer capture interrupt and timer overflow interrupt
}

// this is a wrapper for usb_putchar that allows me to use printf from stdio.h
static inline int debug_putchar(char c, FILE* stream)
{
    return usb_serial_putchar(c); // refer to http://www.pjrc.com/teensy/usb_serial.html
}

// setup stream for printf from stdio.h
static FILE mystdout = FDEV_SETUP_STREAM(debug_putchar, NULL, _FDEV_SETUP_WRITE);

int main()
{    
    clock_prescale_set(clock_div_2); // this is because we are running at 3.3V
    // see http://www.nongnu.org/avr-libc/user-manual/group__avr__power.html and http://www.pjrc.com/teensy/3volt.html
    
    stdout = &mystdout; // setup stream for printf from stdio.h
    
    usb_init(); // initialize usb obviously
    
    while (usb_configured() == 0) _delay_ms(5); // waits until USB is ready
    
    _delay_ms(5000); // 5 seconds for you to connect hyperterminal
    
    tmrcap_init(); // initialize timer input capture
    sei(); // enable interrupts
    
    // here's the start of the demonstration
    printf("Begin\r\n");
    
    while (1); // infinit loop
    return 0;
}